﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Xml.Serialization;

namespace Rlc.PhotoGallery.Bll
{
	[Serializable]
	public class PhotoInfo
	{
		public enum RotateOptions { None, Ninety, OneEighty, TwoSeventy };
		public enum PrivacyOptions { Public, Family, Private };
		public string Title { get; set; }
		public string Tags { get; set; }
		public RotateOptions Rotation { get; set; }
		public PrivacyOptions Privacy { get; set; }

		public PhotoInfo()
		{
			Title = "";
			Tags = "";
			Rotation = RotateOptions.None;
			Privacy = PrivacyOptions.Family;
		}

		#region Static Helper Methods

		/// <summary>
		/// Saves the photos.
		/// </summary>
		/// <param name="photos">The photos.</param>
		/// <returns></returns>
		public static bool SavePhotoMetadata(string photoPath, PhotoInfo photoInfo)
		{
			// Initialize return result.
			//
			bool result = false;

			// Create the path for the metadata file.
			//
			string directory = Path.GetDirectoryName(photoPath);
			string metaFile = Path.GetFileNameWithoutExtension(photoPath) + ".xml";
			metaFile = Path.Combine(directory, metaFile);

			// Open up a file stream for saving.
			//
			Stream fileStream = new FileStream(metaFile, FileMode.Create, FileAccess.Write);

			// Serialize out the data.
			//
			XmlSerializer xmlSerializer = new XmlSerializer(typeof(PhotoInfo));
			try
			{
				xmlSerializer.Serialize(fileStream, photoInfo);
				result = true;
			}
			catch (Exception ex)
			{
			}
			finally
			{
				fileStream.Close();
			}

			return result;
		}

		/// <summary>
		/// Loads the photos.
		/// </summary>
		/// <returns></returns>
		public static PhotoInfo LoadPhotoMetadata(string photoPath)
		{
			// Create the path for the metadata file.
			//
			string directory = Path.GetDirectoryName(photoPath);
			string metaFile = Path.GetFileNameWithoutExtension(photoPath) + ".xml";
			metaFile = Path.Combine(directory, metaFile);

			// Validate the existence of the data file.
			//
			if (!File.Exists(metaFile))
			{
				// File is not there...create a new database.
				//
				return new PhotoInfo();
			}

			// Open a stream for reading.
			//
			Stream fileStream = new FileStream(metaFile, FileMode.Open, FileAccess.Read);

			// Deserialize the data.
			//
			XmlSerializer xmlSerializer = new XmlSerializer(typeof(PhotoInfo));
			PhotoInfo photoInfo = (PhotoInfo)xmlSerializer.Deserialize(fileStream);

			// Close the stream.
			//
			fileStream.Close();

			return photoInfo;
		}

		#endregion
	}
	
	public class GalleryObject
    {
        public enum GalleryObjectTypeOptions { Directory, Image };
        public GalleryObjectTypeOptions GalleryObjectType { get; set; }
        public string FullPath { get; set; }
		public PhotoInfo Info { get; set; }
	}

	public class GalleryObjectCollection : List<GalleryObject>
	{

	}

    public static class GalleryManager
    {
		public static int ThumbNailWidth = 130;
		public static int ThumbNailHeight = 97;

		/// <summary>
		/// Gets the object.
		/// </summary>
		/// <param name="imageFullPath">The image full path.</param>
		/// <returns></returns>
		public static GalleryObject GetObject(string imageFullPath)
		{
			// Validate existance.
			//
			if (!File.Exists(imageFullPath))
			{
				return null;
			}

			return GetObject(imageFullPath, GalleryObject.GalleryObjectTypeOptions.Image);
		}

		/// <summary>
		/// Gets the object.
		/// </summary>
		/// <param name="file">The file.</param>
		/// <returns></returns>
		public static GalleryObject GetObject(FileInfo file)
		{
			return GetObject(file.FullName, GalleryObject.GalleryObjectTypeOptions.Image);
		}

		/// <summary>
		/// Gets the object.
		/// </summary>
		/// <param name="directory">The directory.</param>
		/// <returns></returns>
		public static GalleryObject GetObject(DirectoryInfo directory)
		{
			return GetObject(directory.FullName, GalleryObject.GalleryObjectTypeOptions.Directory);
		}

		/// <summary>
		/// Gets the object.
		/// </summary>
		/// <param name="fullPath">The full path.</param>
		/// <param name="type">The type.</param>
		/// <returns></returns>
		public static GalleryObject GetObject(string fullPath, GalleryObject.GalleryObjectTypeOptions type)
		{
			// Create a gallery object.
			//
			GalleryObject go = new GalleryObject();
			go.GalleryObjectType = type;
			go.FullPath = Path.GetFullPath(fullPath);

			// Load the photo metadata.
			//
			go.Info = PhotoInfo.LoadPhotoMetadata(go.FullPath);

			// Ensure the photo has a title. Photos do not
			//	require metadata. For those photos, just
			//	use the file name.
			//
			if (string.IsNullOrEmpty(go.Info.Title))
			{
				string[] parts = go.FullPath.Split(Path.DirectorySeparatorChar);
				go.Info.Title = parts[parts.Length - 1];
			}

			return go;
		}

        /// <summary>
        /// Gets all objects.
        /// </summary>
        /// <param name="folderName">Name of the folder.</param>
        /// <returns></returns>
		public static GalleryObjectCollection GetAllObjects(string folderName, UserInfo user)
        {
			// Instantiate the return object.
			//
			GalleryObjectCollection results = new GalleryObjectCollection();

			// Validate the folder exists.
			//
			DirectoryInfo di = new DirectoryInfo(folderName);
			if (!di.Exists)
			{
				return results;
			}

			// Enumerate the child folders.
			//
			DirectoryInfo[] subFolders = di.GetDirectories();
			foreach (DirectoryInfo subDi in subFolders)
			{
				// Don't inlcude system folders.
				//
				if (subDi.Attributes != FileAttributes.System)
				{
					if ( IsAuthenticated(subDi, user) )
					{
						// Get the gallery object and add it to the list.
						//
						GalleryObject go = GetObject(subDi);
						results.Add(go);
					}
				}
			}

			// Enumerate the files.
			//
			FileInfo[] files = di.GetFiles();
			foreach (FileInfo fi in files)
			{
				if (IsAuthenticated(fi, user))
				{
					GalleryObject go = GetObject(fi);
					results.Add(go);
				}
			}

			return results;
		}

        /// <summary>
        /// Gets the thumb nail image full path.
        /// </summary>
        /// <param name="obj">The obj.</param>
        /// <param name="thumbRootFolder">The thumb root folder.</param>
        /// <param name="photoRootFolder">The photo root folder.</param>
        /// <returns></returns>
        public static string GetThumbNailImageFullPath(GalleryObject obj, string thumbRootFolder, string photoRootFolder)
        {
			if (obj.GalleryObjectType == GalleryObject.GalleryObjectTypeOptions.Directory)
			{
				return GetFolderImageFullPath(obj.FullPath, thumbRootFolder, photoRootFolder);
			}
			else
			{
				return GetThumbNailImageFullPath(obj.FullPath, (int)obj.Info.Rotation, thumbRootFolder, photoRootFolder);
			}
        }

        /// <summary>
        /// Gets the thumb nail image full path.
        /// </summary>
        /// <param name="imageFullPath">The image full path.</param>
        /// <param name="thumbRootFolder">The thumb root folder.</param>
        /// <param name="photoRootFolder">The photo root folder.</param>
        /// <returns></returns>
		public static string GetThumbNailImageFullPath(string imageFullPath, int rotationIndex, string thumbRootFolder, string photoRootFolder)
        {
			// Use the full image path to generate the path
			//	to the thumbnail.
			//
			string thumbNailFile = imageFullPath.Replace(photoRootFolder, thumbRootFolder);

			// If the thumbnail does not exist...
			//
            if (!File.Exists(thumbNailFile))
            {
				// Create it and return it.
				//
				if (CreateThumbNail(imageFullPath, thumbNailFile, rotationIndex))
                {
                    return thumbNailFile;
                }
                else
                {
                    return null;
                }
            }
            else
            {
				// Return the one that exits.
				//
                return thumbNailFile;
            }
        }

		/// <summary>
		/// Gets the folder image full path.
		/// </summary>
		/// <param name="folderFullPath">The folder full path.</param>
		/// <param name="thumbRootFolder">The thumb root folder.</param>
		/// <param name="photoRootFolder">The photo root folder.</param>
		/// <returns></returns>
		public static string GetFolderImageFullPath(string folderFullPath, string thumbRootFolder, string photoRootFolder)
		{
			// Use the full image path to generate the path
			//	to the folder image (thumbnail).
			//
			string thumbNailFile = folderFullPath.Replace(photoRootFolder, thumbRootFolder);
			thumbNailFile = Path.Combine(thumbNailFile, "folderthumb");

			// If the thumbnail does not exist...
			//
			if (!File.Exists(thumbNailFile))
			{
				// Create it and return it.
				//
				if (CreateFolderImage(folderFullPath, thumbNailFile))
				{
					return thumbNailFile;
				}
				else
				{
					return null;
				}
			}
			else
			{
				return thumbNailFile;
			}
		}

		#region Private Helpers

		/// <summary>
		/// Determines whether the specified directory is authenticated.
		/// </summary>
		/// <param name="directory">The directory.</param>
		/// <param name="user">The user.</param>
		/// <returns>
		/// 	<c>true</c> if the specified directory is authenticated; otherwise, <c>false</c>.
		/// </returns>
		private static bool IsAuthenticated(DirectoryInfo directory, UserInfo user)
		{
			// Public users pay more searching time since there will
			//	be less public images.
			//

			// Return value
			//
			bool isAuthenticated = false;

			// Get all the files in this child directory
			//
			FileInfo[] childFiles = directory.GetFiles();

			// Only include folders that have at least one image
			//	matching this users permissions.
			//
			foreach (FileInfo fi in childFiles)
			{
				isAuthenticated = IsAuthenticated(fi, user);
				if (isAuthenticated)
				{
					// Leave early...only need to find one.
					//
					break;
				}
			}

			if (!isAuthenticated)
			{
				// Could not find any files in this folder.
				//	What about child folders.
				//
				DirectoryInfo[] childrenFolders = directory.GetDirectories();
				foreach (DirectoryInfo childFolder in childrenFolders)
				{
					// Recursively crawl...
					//
					isAuthenticated = IsAuthenticated(childFolder, user);
					if (isAuthenticated)
					{
						// Found one...leave early.
						//
						break;
					}
				}
			}

			return isAuthenticated;
		}

		/// <summary>
		/// Determines whether the specified file is authenticated.
		/// </summary>
		/// <param name="file">The file.</param>
		/// <param name="user">The user.</param>
		/// <returns>
		/// 	<c>true</c> if the specified file is authenticated; otherwise, <c>false</c>.
		/// </returns>
		private static bool IsAuthenticated(FileInfo file, UserInfo user)
		{
			bool isAuthenticated = false;

			// Only show jpg images.
			//
			if (file.Extension.IndexOf("jpg", 0, StringComparison.InvariantCultureIgnoreCase) != -1)
			{
				// Get the gallery object and add it to the list.
				//
				GalleryObject go = GetObject(file);
				if (go.Info.Privacy == PhotoInfo.PrivacyOptions.Private)
				{
					// Only admin users
					//
					if (user == null ||
						user.UserType != UserInfo.TypeOptions.Admin)
					{
						// This user is not authenticated
						//
					}
					else
					{
						// This user is authenticated.
						//
						isAuthenticated = true;
					}
				}
				else if (go.Info.Privacy == PhotoInfo.PrivacyOptions.Family)
				{
					// Only admin or family
					//
					if (user == null ||
						user.UserType == UserInfo.TypeOptions.Public)
					{
						// This user is not authenticated.
						//
					}
					else
					{
						// This user is authenticated.
						//
						isAuthenticated = true;
					}
				}
				else if (go.Info.Privacy == PhotoInfo.PrivacyOptions.Public)
				{
					// Everyone can see this image.
					//
					isAuthenticated = true;
				}
			}

			return isAuthenticated;
		}

		/// <summary>
		/// Creates the thumb nail.
		/// </summary>
		/// <param name="imageFile">The image file.</param>
		/// <param name="thumbNailFile">The thumb nail file.</param>
		/// <returns></returns>
		public static bool CreateThumbNail(string imageFile, string thumbNailFile, int rotationIndex)
		{
			// Create the new image.
			//
			Image thumbNail = CreateThumbNail(imageFile, rotationIndex);

			// Try to save this file.
			//
			try
			{
				thumbNail.Save(thumbNailFile, System.Drawing.Imaging.ImageFormat.Jpeg);
			}
			catch (Exception ex)
			{
				return false;
			}

			return true;
		}

		/// <summary>
		/// Creates the thumb nail.
		/// </summary>
		/// <param name="imageFile">The image file.</param>
		/// <returns></returns>
        private static Image CreateThumbNail(string imageFile, int rotationIndex)
        {
			// Load the full size image.
			//
			System.Drawing.Image fullSizeImg;
			try
			{
				fullSizeImg = new System.Drawing.Bitmap(imageFile);

				// Do the rotation
				//
				if (rotationIndex == 1)
				{
					fullSizeImg.RotateFlip(RotateFlipType.Rotate90FlipNone);
				}
				else if (rotationIndex == 2)
				{
					fullSizeImg.RotateFlip(RotateFlipType.Rotate180FlipNone);
				}
				else if (rotationIndex == 3)
				{
					fullSizeImg.RotateFlip(RotateFlipType.Rotate270FlipNone);
				}
			}
			catch (Exception ex)
			{
				// If we could not load the full size image then exit early
				//
				return null;
			}

			// Create a blank thumbnail image
			//
			Image thumbNailImg = new Bitmap(fullSizeImg, ThumbNailWidth, ThumbNailHeight);

			// Get the GDI drawing surface from the thumbnail image.
			//
			Graphics g = Graphics.FromImage(thumbNailImg);

			// Initialize the drawing surface.
			//
			g.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighQuality;
			g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
			g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;

			// Fill the image with white.
			//
			g.FillRectangle(Brushes.White, 0, 0, ThumbNailWidth, ThumbNailHeight);

			// Create a rectanble the size of the new thumbnail
			//
			Rectangle destination = new Rectangle(0, 0, ThumbNailWidth, ThumbNailHeight);

			// Determine the width and height of the original image that we want
			//	to use to make the thumbnail. Don't take the whole thing. Take the
			//	full width of the original, but only take the height of the original
			//	that allows us to keep the aspect ratio we want.
			//
			int imageWidth = fullSizeImg.Width;
			double widthScale = (double)imageWidth / (double)ThumbNailWidth;
			int imageHeight = (int)((double)ThumbNailHeight * widthScale);

			// Create a rectangle (think sliding view into the original image). The width
			//	will line up with the original. Slide the height to the middle of the image.
			//	Sliding the height to the middle is generally a good choice because most
			//	people will center their content in the image.
			//
			Rectangle source = new Rectangle(0, (fullSizeImg.Height - imageHeight) / 2, imageWidth, imageHeight);

			// Draw the selected portion of the original onto the thumbnail surface.
			//	This will scale the image.
			//
			g.DrawImage(fullSizeImg, destination, source, GraphicsUnit.Pixel);

			// Clean up gdi resources.
			//
			g.Dispose();

			return thumbNailImg;
        }

		/// <summary>
		/// Creates the folder image.
		/// </summary>
		/// <param name="folderName">Name of the folder.</param>
		/// <param name="folderImageFile">The folder image file.</param>
		/// <returns></returns>
		private static bool CreateFolderImage(string folderName, string folderImageFile)
		{
			// If the folder image does not exist, then
			//	the thumbnail folder for this folder does not
			//	exist.
			//
			string folder = Path.GetDirectoryName(folderImageFile);
			if (!Directory.Exists(folder))
			{
				Directory.CreateDirectory(folder);
			}

			// Create the new image.
			//
			Image thumbNail = CreateFolderImage( false, folderName);

			// Try to save this file.
			//
			try
			{
				thumbNail.Save(folderImageFile, System.Drawing.Imaging.ImageFormat.Png);
			}
			catch (Exception ex)
			{
				return false;
			}

			return true;
		}

		/// <summary>
		/// Creates the thumbnail Bitmap for a folder.
		/// </summary>
		/// <param name="isParentFolder">True if the up arrow must be drawn.</param>
		/// <param name="folder">The path of the folder.</param>
		/// <param name="size">The size in pixels of a square bounding the thumbnail.</param>
		/// <returns>The thumbnail Bitmap.</returns>
		private static Image CreateFolderImage(bool isParentFolder, string folder)
		{
			// Set the return image size.
			//
			int size = ThumbNailWidth;

			// Create the return image.
			//
			Image newImage = new Bitmap(size, size, System.Drawing.Imaging.PixelFormat.Format32bppArgb);

			// Get the graphics surface and initialize.
			//
			Graphics g = Graphics.FromImage(newImage);
			g.InterpolationMode = InterpolationMode.HighQualityBicubic;
			g.SmoothingMode = SmoothingMode.HighQuality;
			g.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighQuality;

			// Fill the image with white but fully transparent.
			//
			g.Clear(Color.Transparent);

			// Create a color matrix
			//
			ColorMatrix cm = new ColorMatrix();
			cm.Matrix00 = 1;
			cm.Matrix11 = 1;
			cm.Matrix22 = 1;
			cm.Matrix33 = 0.5f;

			// Generate a list of images to draw.
			//
			int numPicturesInStack = 3;
			List<string> imagesToDraw = new List<string>();

			// Look for default images first.
			//
			string[] images = Directory.GetFiles(folder, "*default.jpg", SearchOption.AllDirectories);
			int numToUse = Math.Min(numPicturesInStack, images.Length);
			for (int i = 0; i < numToUse; i++)
			{
				imagesToDraw.Insert(0, images[i]);
			}

			// If not enough default images to fill the stack...
			//
			if (imagesToDraw.Count < numPicturesInStack)
			{
				// Use any image to fill the stack.
				//
				images = Directory.GetFiles(folder, "*.jpg", SearchOption.AllDirectories);
				numToUse = Math.Min(numPicturesInStack - imagesToDraw.Count, images.Length);
				for (int i = 0; i < numToUse; i++)
				{
					imagesToDraw.Insert(0, images[i]);
				}
			}

			// Initialize some parameters that will be used
			//	in the main drawing loop.
			//
			int drawXOffset = size / 2;
			int drawYOffset = size / 2;
			double angleAmplitude = Math.PI / 10;
			int imageFolderSize = (int)(size / (Math.Cos(angleAmplitude) + Math.Sin(angleAmplitude)));
			Color folderBorderColor = Color.FromArgb(90, Color.Black);
			float folderBorderWidth = 2;
			Random rnd = new Random();

			// Draw each image in the stack.
			//
			foreach (string folderImagePath in imagesToDraw)
			{
				// Read the original image.
				//
				Bitmap folderImage = new Bitmap(folderImagePath);

				// Get the width and height of the original image.
				//
				int width = folderImage.Width;
				int height = folderImage.Height;

				if (imageFolderSize > 0 && folderImage.Width >= folderImage.Height && folderImage.Width > imageFolderSize)
				{
					// Landscape mode picture....scale the height to fit.
					//
					width = imageFolderSize;
					height = imageFolderSize * folderImage.Height / folderImage.Width;
				}
				else if (imageFolderSize > 0 && folderImage.Height >= folderImage.Width && folderImage.Height > imageFolderSize)
				{
					// Portrait mode picture...scale the width to fit.
					//
					width = imageFolderSize * folderImage.Width / folderImage.Height;
					height = imageFolderSize;
				}

				// Randomize the tilt angle.
				//
				double angle = (0.5 - rnd.NextDouble()) * angleAmplitude;

				// Create a pen to draw the border.
				//
				Pen borderPen = new Pen(new SolidBrush(folderBorderColor), folderBorderWidth);
				borderPen.LineJoin = LineJoin.Round;
				borderPen.StartCap = LineCap.Round;
				borderPen.EndCap = LineCap.Round;

				// Calculae the geometrical transformations necessary
				//	for drawing.
				//
				float sin = (float)Math.Sin(angle);
				float cos = (float)Math.Cos(angle);
				float sh = sin * height / 2;
				float ch = cos * height / 2;
				float sw = sin * width / 2;
				float cw = cos * width / 2;
				float shb = sin * (height + borderPen.Width) / 2;
				float chb = cos * (height + borderPen.Width) / 2;
				float swb = sin * (width + borderPen.Width) / 2;
				float cwb = cos * (width + borderPen.Width) / 2;

				// Draw the border.
				//
				g.DrawPolygon(borderPen, new PointF[] {
					new PointF(
						(float)drawXOffset - cwb - shb,
						(float)drawYOffset + chb - swb),
					new PointF(
						(float)drawXOffset - cwb + shb,
						(float)drawYOffset - swb - chb),
					new PointF(
						(float)drawXOffset + cwb + shb,
						(float)drawYOffset + swb - chb),
					new PointF(
						(float)drawXOffset + cwb - shb,
						(float)drawYOffset + swb + chb)
				});

				// Draw the current image.
				//
				g.DrawImage(folderImage, new PointF[] {
					new PointF(
						(float)drawXOffset - cw + sh,
						(float)drawYOffset - sw - ch),
					new PointF(
						(float)drawXOffset + cw + sh,
						(float)drawYOffset + sw - ch),
					new PointF(
						(float)drawXOffset - cw - sh,
						(float)drawYOffset + ch - sw)
				});

				// Free the gdi resources.
				//
				folderImage.Dispose();
				borderPen.Dispose();
			}

			// If necessary draw the navigation arrow.
			//
			if (isParentFolder)
			{
				// Set the qualities that define the arrow.
				//
				Color arrowColor = Color.FromArgb(200, Color.Beige);
				float arrowWidth = 4;
				float arrowSize = 0.125F;

				// Create a pen to draw the arrow.
				//
				Pen arrowPen = new Pen(new SolidBrush(arrowColor), arrowWidth);
				arrowPen.LineJoin = LineJoin.Round;
				arrowPen.StartCap = LineCap.Flat;
				arrowPen.EndCap = LineCap.Round;

				g.DrawLines(arrowPen, new PointF[] {
					new PointF((float)drawXOffset - arrowSize * size, (float)drawYOffset + arrowSize * size),
					new PointF((float)drawXOffset + arrowSize * size, (float)drawYOffset + arrowSize * size),
					new PointF((float)drawXOffset + arrowSize * size, (float)drawYOffset - arrowSize * size),
					new PointF((float)drawXOffset + arrowSize * size * 2 / 3, (float)drawYOffset - arrowSize * size * 2 / 3),
					new PointF((float)drawXOffset + arrowSize * size, (float)drawYOffset - arrowSize * size),
					new PointF((float)drawXOffset + arrowSize * size * 4 / 3, (float)drawYOffset - arrowSize * size * 2 / 3)
				});

				// Clean up gdi resources.
				//
				arrowPen.Dispose();
			}

			// Clean up gdi resources.
			//
			g.Dispose();

			return newImage;
		}

		#endregion
	}
}
