﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Xml.Serialization;

namespace Rlc.PhotoGallery.Bll
{

	/// <summary>
	/// User information.
	/// </summary>
	[Serializable]
	public class UserInfo
	{
		public enum TypeOptions { Public, Family, Admin };
		public Guid Id { get; set; }
		public string UserName { get; set; }
		public string Email { get; set; }
		public string Password { get; set; }
		public TypeOptions UserType { get; set; }
	}

	/// <summary>
	/// A collection of users.
	/// </summary>
	[Serializable]
	public class UserInfoCollection : List<UserInfo>
	{
	}

	/// <summary>
	/// Summary description for UserManager
	/// </summary>
	public static class UserManager
	{
		#region Public Helper Methods

		/// <summary>
		/// Gets or sets the user XML file path.
		/// </summary>
		/// <value>The user XML file path.</value>
		public static string UserXmlFilePath { get; set; }

		/// <summary>
		/// Gets the user.
		/// </summary>
		/// <param name="userName">Name of the user.</param>
		/// <param name="passWord">The pass word.</param>
		/// <returns></returns>
		public static UserInfo GetUser(string userName, string passWord)
		{
			UserInfoCollection users = LoadUsers();
			foreach (UserInfo user in users)
			{
				if (user.UserName == userName && user.Password == passWord)
				{
					return user;
				}
			}
			return null;
		}

		#endregion

		#region Public CRUD methods

		/// <summary>
		/// Adds the user.
		/// </summary>
		/// <param name="userInfo">The user info.</param>
		/// <returns></returns>
		public static bool AddUser(UserInfo userInfo)
		{
			// Get the current book list.
			//
			UserInfoCollection users = LoadUsers();

			// Add the new book to the list.
			//
			users.Add(userInfo);

			// Write the modified collections.
			//
			return SaveUsers(users);
		}

		/// <summary>
		/// Gets the user.
		/// </summary>
		/// <param name="id">The id.</param>
		/// <returns></returns>
		public static UserInfo GetUser(Guid id)
		{
			UserInfoCollection users = LoadUsers();
			foreach (UserInfo user in users)
			{
				if (user.Id == id)
				{
					return user;
				}
			}
			return null;
		}

		/// <summary>
		/// Deletes the user.
		/// </summary>
		/// <param name="id">The id.</param>
		/// <returns></returns>
		public static bool DeleteUser(Guid id)
		{
			UserInfoCollection users = LoadUsers();
			UserInfo userToDelete = null;
			foreach (UserInfo user in users)
			{
				if (user.Id == id)
				{
					userToDelete = user;
					break;
				}
			}
			if (userToDelete != null)
			{
				if (users.Remove(userToDelete))
				{
					return SaveUsers(users);
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}

		/// <summary>
		/// Updates the user.
		/// </summary>
		/// <param name="userInfo">The user info.</param>
		/// <returns></returns>
		public static bool UpdateUser(UserInfo userInfo)
		{
			UserInfoCollection users = LoadUsers();
			UserInfo userToUpdate = null;
			foreach (UserInfo user in users)
			{
				if (user.Id == userInfo.Id)
				{
					userToUpdate = user;
					break;
				}
			}
			if (userToUpdate != null)
			{
				if (users.Remove(userToUpdate))
				{
					users.Add(userInfo);
					return SaveUsers(users);
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}

		#endregion

		#region Private Methods

		/// <summary>
		/// Loads the users.
		/// </summary>
		/// <returns></returns>
		private static UserInfoCollection LoadUsers()
		{
			// Validate the xml file path
			//
			if (string.IsNullOrEmpty(UserXmlFilePath))
			{
				throw new Exception("Path to user xml data file not set!");
			}

			// Validate the existence of the data file.
			//
			if (!File.Exists(UserXmlFilePath))
			{
				// File is not there...create the default
				//	users.
				return GenerateSampleUsers();
			}

			// Open a stream for reading.
			//
			Stream fileStream = new FileStream(UserXmlFilePath, FileMode.Open, FileAccess.Read);

			// Deserialize the data.
			//
			XmlSerializer xmlSerializer = new XmlSerializer(typeof(UserInfoCollection));
			UserInfoCollection users = (UserInfoCollection)xmlSerializer.Deserialize(fileStream);

			// Close the stream.
			//
			fileStream.Close();

			// Sort by descending dates.
			//
			users.Sort(delegate(UserInfo u1, UserInfo u2) { return u1.UserName.CompareTo(u2.UserName); });

			return users;
		}

		/// <summary>
		/// Saves the users.
		/// </summary>
		/// <param name="users">The users.</param>
		/// <returns></returns>
		private static bool SaveUsers(UserInfoCollection users)
		{
			// Validate the xml file path
			//
			if (string.IsNullOrEmpty(UserXmlFilePath))
			{
				throw new Exception("Path to user xml data file not set!");
			}

			// Initialize return result.
			//
			bool result = false;

			// Open up a file stream for saving.
			//
			Stream fileStream = new FileStream(UserXmlFilePath, FileMode.Create, FileAccess.Write);

			// Serialize out the data.
			//
			XmlSerializer xmlSerializer = new XmlSerializer(typeof(UserInfoCollection));
			try
			{
				xmlSerializer.Serialize(fileStream, users);
				result = true;
			}
			catch (Exception ex)
			{
			}
			finally
			{
				fileStream.Close();
			}

			return result;
		}

		/// <summary>
		/// Generates the sample users.
		/// </summary>
		private static UserInfoCollection GenerateSampleUsers()
		{
			UserInfo admin = new UserInfo();
			admin.Id = Guid.NewGuid();
			admin.UserName = "admin";
			admin.UserType = UserInfo.TypeOptions.Admin;
			admin.Password = "admin";
			admin.Email = "x@y.com";

			UserInfo family = new UserInfo();
			family.Id = Guid.NewGuid();
			family.UserName = "family";
			family.UserType = UserInfo.TypeOptions.Family;
			family.Password = "family";
			family.Email = "x@y.com";

			UserInfo pub = new UserInfo();
			pub.Id = Guid.NewGuid();
			pub.UserName = "public";
			pub.UserType = UserInfo.TypeOptions.Public;
			pub.Password = "public";
			pub.Email = "x@y.com";

			UserInfoCollection users = new UserInfoCollection();
			users.Add(admin);
			users.Add(family);
			users.Add(pub);

			SaveUsers(users);

			return users;
		}

		#endregion
	}
}