﻿using System.Web.Script.Services;
using System.Web.Services;
using Rlc.PhotoGallery.Bll;
using System;
using System.Web;
using System.IO;
using System.Web.UI.WebControls;
using System.Text;
using System.Web.UI;
using System.Collections.Generic;
using System.Web.UI.HtmlControls;
using System.Reflection;
using Rlc.Utilities.Logging;

namespace Rlc.PhotoGallery.Web
{
	/// <summary>
	/// This class provides a service layer to expose a 
	/// number of business layer object and data to the
	/// presentation layer. A number of these methods are
	/// callable via JavaScript.
	/// </summary>
	[WebService(Namespace = "http://bobcravens.com/photogallery/")]
	[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
	[ScriptService] // Make these calls available to JavaScript.
	public class ServiceLayer : WebService
	{
		[WebMethod(EnableSession = true)]
		public string GetThumbList(string relativePath)
		{
			// Create the full path.
			//
			string rootDirectory = ConfigData.RootGalleryDirectory;
			string fullPath = Path.Combine(rootDirectory, relativePath);
			fullPath = Path.GetFullPath(fullPath);

			// Get all the object contained in this folder.
			//
			List<GalleryObject> allItems = GalleryManager.GetAllObjects(fullPath, CurrentUser);

			return BuildThumbListString(allItems);
		}

		private string BuildThumbListString(List<GalleryObject> items)
		{

			// Add each item to the page.
			//
			StringBuilder imgList = new StringBuilder();
			foreach (GalleryObject item in items)
			{
				if (imgList.Length > 0)
				{
					imgList.Append("|");
				}
				string relImgPath = item.FullPath.Replace(ConfigData.RootGalleryDirectory, "");
				string rotation = ":0";
				string privacy = ":Public";
				if (item.GalleryObjectType == GalleryObject.GalleryObjectTypeOptions.Directory)
				{
					imgList.Append("d:");
					relImgPath += "/folderthumb";
				}
				else
				{
					imgList.Append("i:");
					rotation = string.Format(":{0}", (int)item.Info.Rotation);
					privacy = ":" + item.Info.Privacy.ToString();
				}

				imgList.Append(relImgPath);

				imgList.Append(rotation);

				imgList.Append(privacy);
			}

			return imgList.ToString();
		}

		[WebMethod(EnableSession = true)]
		public string DoSearch(string searchString)
		{
			// Get key words
			//
			string[] keys = searchString.Split(' ');

			// Create the full path.
			//
			string rootDirectory = ConfigData.RootGalleryDirectory;

			// Get all the object contained in this folder.
			//
			List<GalleryObject> matchedItems = Search(rootDirectory, keys);

			return BuildThumbListString(matchedItems);
		}

		/// <summary>
		/// Recursive searches the specified directory.
		/// </summary>
		/// <param name="directory">The directory.</param>
		/// <param name="keys">The keys.</param>
		/// <returns></returns>
		private List<GalleryObject> Search(string directory, string[] keys)
		{
			// This algorithm could use some caching. For instance,
			//	key words will regularly be reused (someone's name
			//	for instance). The results of the previous search
			//	could be cached along with the range of pictures
			//	that were previously searched. The search could
			//	continue against only new pictures and then the cache
			//	data updated.

			List<GalleryObject> matchedItems = new List<GalleryObject>();
			List<GalleryObject> rootItems = GalleryManager.GetAllObjects(directory, CurrentUser);
			foreach (GalleryObject go in rootItems)
			{
				if (go.GalleryObjectType == GalleryObject.GalleryObjectTypeOptions.Image)
				{
					foreach (string key in keys)
					{
						if (go.Info.Title.IndexOf(key, StringComparison.CurrentCultureIgnoreCase) != -1)
						{
							// Found a key word in the title.
							//
							matchedItems.Add(go);
							break;
						}
						if (go.Info.Tags.IndexOf(key, StringComparison.CurrentCultureIgnoreCase) != -1)
						{
							// Found a key word in one of the tags.
							//
							matchedItems.Add(go);
							break;
						}
						if (go.FullPath.IndexOf(key, StringComparison.CurrentCultureIgnoreCase) != -1)
						{
							// Found a key word in one of the folder names.
							//
							matchedItems.Add(go);
							break;
						}
					}
				}
				else
				{
					matchedItems.AddRange(Search(go.FullPath, keys));
				}
			}

			return matchedItems;
		}

		[WebMethod(EnableSession = true)]
		public string GetThumbHtml(string relativePath)
		{
			// Create the full path.
			//
			string rootDirectory = ConfigData.RootGalleryDirectory;
			string fullPath = Path.Combine(rootDirectory, relativePath);
			fullPath = Path.GetFullPath(fullPath);

			// Create a page object to faciliate the rendering.
			//
			Page page = new Page();

			// Create a place holder to hold the controls
			//
			PlaceHolder placeHolderThumbs = new PlaceHolder();
			page.Controls.Add(placeHolderThumbs);

			// Get all the object contained in this folder.
			//
			List<GalleryObject> allItems = GalleryManager.GetAllObjects(fullPath, CurrentUser);

			// Add each item to the page.
			//
			foreach (GalleryObject item in allItems)
			{
				// Create a user control for this item.
				//
				try
				{
					UserControl imageControl = page.LoadControl("ImageFrame.ascx") as UserControl;
					Type imageControlType = imageControl.GetType();
					FieldInfo field = imageControlType.GetField("Item");
					if (field != null)
					{
						field.SetValue(imageControl, item);
					}
					placeHolderThumbs.Controls.Add(imageControl);
				}
				catch (Exception ex)
				{
				}
			}

			StringWriter output = new StringWriter();
			HttpContext.Current.Server.Execute(page, output, false);

			return output.ToString() + "|" + GetPathHtml(relativePath);
		}

		/// <summary>
		/// Gets the path HTML.
		/// </summary>
		/// <param name="relativePath">The relative path.</param>
		/// <returns></returns>
		[WebMethod(EnableSession = true)]
		public string GetPathHtml(string relativePath)
		{
			// Create the full path.
			//
			string rootDirectory = ConfigData.RootGalleryDirectory;
			string fullPath = Path.Combine(rootDirectory, relativePath);
			fullPath = Path.GetFullPath(fullPath);

			// Create a place holder to hold the controls
			//
			PlaceHolder placeHolderCurrentPath = new PlaceHolder();

			// Strip off the root directory.
			//
			List<string> allFolders = new List<string>();
			allFolders.Add("root");
			if (!string.IsNullOrEmpty(relativePath))
			{
				string[] folders = relativePath.Replace('/', '\\').Split(Path.DirectorySeparatorChar);
				allFolders.AddRange(folders);
			}
			string currentFolder = string.Empty;
			for (int i = 0; i < allFolders.Count; i++)
			{
				if (string.IsNullOrEmpty(allFolders[i]))
				{
					continue;
				}
				if (i == 0 && allFolders.Count != 1)
				{
					// Add the link to the root folder.
					//
					HtmlAnchor link = new HtmlAnchor();
					link.InnerHtml = allFolders[i];
					link.Attributes.Add("onclick", "loadThumbs('');return false;");
					link.HRef = "javascript::void();";
					placeHolderCurrentPath.Controls.Add(link);
				}
				else if (i < allFolders.Count - 1)
				{
					// Add the link to the folder.
					//
					currentFolder = Path.Combine(currentFolder, allFolders[i]);
					HtmlAnchor link = new HtmlAnchor();
					link.InnerHtml = allFolders[i];
					link.Attributes.Add("onclick", "loadThumbs('" + currentFolder.Replace('\\', '/') + "');return false;");
					link.HRef = "javascript::void();";
					placeHolderCurrentPath.Controls.Add(link);
				}
				else
				{
					// Don't link to myself...I'm already there.
					//
					Label label = new Label();
					label.Text = allFolders[i];
					placeHolderCurrentPath.Controls.Add(label);
				}

				Label seperator = new Label();
				seperator.Text = "\\";
				placeHolderCurrentPath.Controls.Add(seperator);
			}

			StringBuilder sb = new StringBuilder();
			StringWriter tw = new StringWriter(sb);
			HtmlTextWriter hw = new HtmlTextWriter(tw);
			placeHolderCurrentPath.RenderControl(hw);
			return sb.ToString();
		}
	
		/// <summary>
		/// Sets the tags.
		/// </summary>
		/// <param name="imageUrl">The image URL.</param>
		/// <param name="newTags">The new tags.</param>
		/// <returns></returns>
		[WebMethod(EnableSession = true)]
		public string SetTags(string imageUrl, string newTags)
		{
			if (IsAdmin)
			{
				string imageFullPath = GetImageFullPath(imageUrl);
				if (!string.IsNullOrEmpty(imageFullPath))
				{
					GalleryObject go = GalleryManager.GetObject(imageFullPath);
					go.Info.Tags = newTags;
					if (!PhotoInfo.SavePhotoMetadata(go.FullPath, go.Info))
					{
						throw new Exception("Failed to set privacy!");
					}
				}
				return GetImageInfo(imageUrl);
			}
			else
			{
				throw new Exception("operation failed");
			}
		}

		/// <summary>
		/// Sets the title.
		/// </summary>
		/// <param name="imageUrl">The image URL.</param>
		/// <param name="newTitle">The new title.</param>
		/// <returns></returns>
		[WebMethod(EnableSession = true)]
		public string SetTitle(string imageUrl, string newTitle)
		{
			if (IsAdmin)
			{
				string imageFullPath = GetImageFullPath(imageUrl);
				if (!string.IsNullOrEmpty(imageFullPath))
				{
					GalleryObject go = GalleryManager.GetObject(imageFullPath);
					go.Info.Title = newTitle;
					if (!PhotoInfo.SavePhotoMetadata(go.FullPath, go.Info))
					{
						throw new Exception("Failed to set privacy!");
					}
				}
				return GetImageInfo(imageUrl);
			}
			else
			{
				throw new Exception("operation failed");
			}
		}

		/// <summary>
		/// Rotates the CW.
		/// </summary>
		/// <param name="imageUrl">The image URL.</param>
		/// <returns></returns>
		[WebMethod(EnableSession = true)]
		public string RotateCW(string imageUrl)
		{
			if (IsAdmin)
			{
				string imageFullPath = GetImageFullPath(imageUrl);
				if (!string.IsNullOrEmpty(imageFullPath))
				{
					GalleryObject go = GalleryManager.GetObject(imageFullPath);
					if (go.Info.Rotation == PhotoInfo.RotateOptions.None)
					{
						go.Info.Rotation = PhotoInfo.RotateOptions.Ninety;
					}
					else if (go.Info.Rotation == PhotoInfo.RotateOptions.Ninety)
					{
						go.Info.Rotation = PhotoInfo.RotateOptions.OneEighty;
					}
					else if (go.Info.Rotation == PhotoInfo.RotateOptions.OneEighty)
					{
						go.Info.Rotation = PhotoInfo.RotateOptions.TwoSeventy;
					}
					else
					{
						go.Info.Rotation = PhotoInfo.RotateOptions.None;
					}
					if (!PhotoInfo.SavePhotoMetadata(go.FullPath, go.Info))
					{
						throw new Exception("Failed to set privacy!");
					}
					else
					{
						CreateNewThumb(go);
					}
				}
				return GetImageInfo(imageUrl);
			}
			else
			{
				throw new Exception("operation failed");
			}
		}

		/// <summary>
		/// Rotates the CCW.
		/// </summary>
		/// <param name="imageUrl">The image URL.</param>
		/// <returns></returns>
		[WebMethod(EnableSession = true)]
		public string RotateCCW(string imageUrl)
		{
			if (IsAdmin)
			{
				string imageFullPath = GetImageFullPath(imageUrl);
				if (!string.IsNullOrEmpty(imageFullPath))
				{
					GalleryObject go = GalleryManager.GetObject(imageFullPath);
					if (go.Info.Rotation == PhotoInfo.RotateOptions.None)
					{
						go.Info.Rotation = PhotoInfo.RotateOptions.TwoSeventy;
					}
					else if (go.Info.Rotation == PhotoInfo.RotateOptions.TwoSeventy)
					{
						go.Info.Rotation = PhotoInfo.RotateOptions.OneEighty;
					}
					else if (go.Info.Rotation == PhotoInfo.RotateOptions.OneEighty)
					{
						go.Info.Rotation = PhotoInfo.RotateOptions.Ninety;
					}
					else
					{
						go.Info.Rotation = PhotoInfo.RotateOptions.None;
					}
					if (!PhotoInfo.SavePhotoMetadata(go.FullPath, go.Info))
					{
						throw new Exception("Failed to set privacy!");
					}
					else
					{
						CreateNewThumb(go);
					}
				}
				return GetImageInfo(imageUrl);
			}
			else
			{
				throw new Exception("operation failed");
			}
		}

		/// <summary>
		/// Gets the image info.
		/// </summary>
		/// <param name="imageUrl">The image URL.</param>
		/// <returns></returns>
		[WebMethod(EnableSession = true)]
		public string GetImageInfo(string imageUrl)
		{
			try
			{
				string imageFullPath = GetImageFullPath(imageUrl);
				if (!string.IsNullOrEmpty(imageFullPath))
				{
					GalleryObject go = GalleryManager.GetObject(imageFullPath);
					if (go == null)
					{
						return "Error! No go...path=" + imageFullPath;
					}
					string imageInfo = go.Info.Title;
					if (!string.IsNullOrEmpty(go.Info.Tags))
					{
						imageInfo += " [" + go.Info.Tags + "]";
					}
					if (CurrentUser != null && CurrentUser.UserType == UserInfo.TypeOptions.Admin)
					{
						imageInfo += " - " + go.Info.Privacy.ToString() + ", " + go.Info.Rotation.ToString();
					}
					return imageInfo;
				}
				return string.Empty;
			}
			catch (Exception ex)
			{
				return ex.ToString();
			}
		}

		/// <summary>
		/// Sets the privacy public.
		/// </summary>
		/// <param name="imageUrl">The image URL.</param>
		/// <returns></returns>
		[WebMethod(EnableSession = true)]
		public string SetPrivacyPublic(string imageUrl)
		{
			if (IsAdmin)
			{
				string imageFullPath = GetImageFullPath(imageUrl);
				if (!string.IsNullOrEmpty(imageFullPath))
				{
					GalleryObject go = GalleryManager.GetObject(imageFullPath);
					go.Info.Privacy = PhotoInfo.PrivacyOptions.Public;
					if (!PhotoInfo.SavePhotoMetadata(go.FullPath, go.Info))
					{
						throw new Exception("Failed to set privacy!");
					}
				}
				return GetImageInfo(imageUrl);
			}
			else
			{
				throw new Exception("operation failed");
			}
		}

		/// <summary>
		/// Sets the privacy family.
		/// </summary>
		/// <param name="imageUrl">The image URL.</param>
		/// <returns></returns>
		[WebMethod(EnableSession = true)]
		public string SetPrivacyFamily(string imageUrl)
		{
			if (IsAdmin)
			{
				string imageFullPath = GetImageFullPath(imageUrl);
				if (!string.IsNullOrEmpty(imageFullPath))
				{
					GalleryObject go = GalleryManager.GetObject(imageFullPath);
					go.Info.Privacy = PhotoInfo.PrivacyOptions.Family;
					if (!PhotoInfo.SavePhotoMetadata(go.FullPath, go.Info))
					{
						throw new Exception("Failed to set privacy!");
					}
				}
				return GetImageInfo(imageUrl);
			}
			else
			{
				throw new Exception("operation failed");
			}
		}

		/// <summary>
		/// Sets the privacy private.
		/// </summary>
		/// <param name="imageUrl">The image URL.</param>
		/// <returns></returns>
		[WebMethod(EnableSession = true)]
		public string SetPrivacyPrivate(string imageUrl)
		{
			if (IsAdmin)
			{
				string imageFullPath = GetImageFullPath(imageUrl);
				if (!string.IsNullOrEmpty(imageFullPath))
				{
					GalleryObject go = GalleryManager.GetObject(imageFullPath);
					go.Info.Privacy = PhotoInfo.PrivacyOptions.Private;
					if (!PhotoInfo.SavePhotoMetadata(go.FullPath, go.Info))
					{
						throw new Exception("Failed to set privacy!");
					}
				}
				return GetImageInfo(imageUrl);
			}
			else
			{
				throw new Exception("operation failed");
			}
		}

		/// <summary>
		/// Logins the specified user name.
		/// </summary>
		/// <param name="userName">Name of the user.</param>
		/// <param name="passWord">The pass word.</param>
		/// <returns></returns>
		[WebMethod(EnableSession = true)]
		public string Login(string userName, string passWord)
		{
			UserInfo userInfo = UserManager.GetUser(userName, passWord);
			if (userInfo != null)
			{
				Logger.LogStatus("Security", "Login: " + userInfo.UserType.ToString() + ": " + userInfo.UserName + " (" + userInfo.Email + ")");
				HttpContext.Current.Session["user"] = userInfo;
			}
			else
			{
				Logger.LogWarning("Security", "Login: Invalid Login Attempt - userName='" + userName + "', passWord = '" + passWord + "'");
			}

			return GetUserType();
		}

		/// <summary>
		/// Logouts this instance.
		/// </summary>
		/// <returns></returns>
		[WebMethod(EnableSession = true)]
		public string Logout()
		{
			HttpContext.Current.Session.Remove("user");

			return GetUserType();
		}

		/// <summary>
		/// Gets the type of the user.
		/// </summary>
		/// <returns></returns>
		[WebMethod(EnableSession = true)]
		public string GetUserType()
		{
			UserInfo userInfo = CurrentUser;
			if (userInfo == null)
			{
				return UserInfo.TypeOptions.Public.ToString();
			}
			else
			{
				return userInfo.UserType.ToString();
			}
		}

		/// <summary>
		/// Gets the current user.
		/// </summary>
		/// <value>The current user.</value>
		private UserInfo CurrentUser
		{
			get
			{
				if (HttpContext.Current.Session["user"] != null)
				{
					return (UserInfo)HttpContext.Current.Session["user"];
				}
				return null;
			}
		}

		/// <summary>
		/// Gets the image full path.
		/// </summary>
		/// <param name="imageUrl">The image URL.</param>
		/// <returns></returns>
		private string GetImageFullPath(string imageUrl)
		{
			string[] parts = imageUrl.Split('?');
			if (parts.Length != 2)
			{
				throw new Exception("Unknown query string format!");
			}
			string queryString = parts[1];
			string[] queryParams = queryString.Split('&');
			string relativePath = string.Empty;
			foreach (string queryParam in queryParams)
			{
				parts = queryParam.Split('=');
				if (parts.Length == 2)
				{
					if (parts[0].Equals("rp", StringComparison.CurrentCultureIgnoreCase))
					{
						relativePath = parts[1];
						relativePath = HttpContext.Current.Server.UrlDecode(relativePath);
						break;
					}
				}
			}
			if (string.IsNullOrEmpty(relativePath))
			{
				throw new Exception("Could not find image relative path!");
			}
			string imageFullPath = Path.Combine(ConfigData.RootGalleryDirectory, relativePath);
			imageFullPath = Path.GetFullPath(imageFullPath);
			return imageFullPath;
		}

		/// <summary>
		/// Gets a value indicating whether this instance is admin.
		/// </summary>
		/// <value><c>true</c> if this instance is admin; otherwise, <c>false</c>.</value>
		private bool IsAdmin
		{
			get
			{
				bool isAdmin = false;
				UserInfo userInfo = CurrentUser;
				if (userInfo != null && userInfo.UserType == UserInfo.TypeOptions.Admin)
				{
					isAdmin = true;
				}
				else
				{
					Logger.LogWarning("Security", "Attempted to access admin features without admin rights!");
				}
				return isAdmin;
			}
		}

		private void CreateNewThumb(GalleryObject go)
		{
			string thumbNailFile = GalleryManager.GetThumbNailImageFullPath(go, ConfigData.RootThumbDirectory, ConfigData.RootGalleryDirectory);
			if (File.Exists(thumbNailFile))
			{
				GalleryManager.CreateThumbNail(go.FullPath, thumbNailFile, (int)go.Info.Rotation);
			}
		}
	}
}